/* hash.c
 * Implantation d'une table de double de hachage
 * pour enregistrer des noms avec des dates de naissance
 * Auteur : VOURIOT Thierry
 * Date : 15 decembre 2001
 */

#include "hash.h"

/* Fonction de hachage */
BNat h(BNat i)
{
	double nb = (sqrt(5)-1)/2;
	nb = nb * i;
	nb = nb - (BNat)nb;
	nb = nb * M;
	return (BNat) nb;
}


/* Deuxieme fonction de hachage de type modulo */
BNat pprim(BNat i)
{
	return i%M;
}
/* Deuxieme fonction de hachage de type modulo */
BNat pp(BNat i)
{
	return (2*pprim(i) + 1);
}

/* Creation d'une nouvelle table */
Table tablenouv(void)
{
	int i;
	Table t = MALLOCN(Personne,M);
	for (i=0;i<M;i++)
		t[i].etat = 2;
	return t;
}

/* Adjonction d'une entree dans la table 
 * pre adj(t,i,x) = (elem(t,i)== et x!=)
 */
Table adj(Table t, char *n, int date)
{
	int m;
	Bool ok = false;
	BNat i = convert(n);
	BNat s = h(i);
	Personne p;
	p.etat = 1;
	
	for (m=0;!ok && m<M;m++)
	{
		if (t[s].etat==0 || t[s].etat==2)
		{
			strcpy(p.nom,n); 
			p.naissance = date;
			t[s] = p;
			ok = true;
		}
		else s = (s + pp(i))%M;
	}
	return t;
}

/* Date de naissance en fonction d'un nom */
int elem(Table t, char * n)
{
	int m;
	Bool ok = false;
	BNat i = convert(n);
	BNat s = h(i);
	for (m=0;!ok && m<M;m++)
	{
		if (t[s].etat==1 && (strcmp(t[s].nom,n) == 0))	
			ok = true;
		else if (t[s].etat == 2) 
			return -1;
		else s = (s + pp(i))%M;
	}
	
	if (ok)
		return t[s].naissance;
	else return -1;
}

/* Suppression d'une entree dans la table */
Table supp(Table t, char *n)
{
	int m;
	Bool ok = false;
	BNat i = convert(n);
	BNat s = h(i);
	for (m=0;!ok && m<M;m++)
	{
		if (t[s].etat==1 && (strcmp(t[s].nom,n) == 0))	
		{
			t[s].etat = 0;
			strcpy(t[s].nom,"\0");
			t[s].naissance = 0;
			ok=true;
		}
		else if (t[s].etat == 2) 
			break;
		else s = (s + pp(i))%M;
	}
	
	return t;
}

/* Affichage de la table */
void aff(Table t)
{
	int i;
	int tmp;
	printf("\ni\tnom                    \t   date\t   etat\n");
	printf("-----------------------------------------------\n");
	for (i=0;i<M;i++) 
	{
		printf("%d | \t%s",i,t[i].nom);
		tmp = 20 - strlen(t[i].nom);
		while (tmp != 0) { printf(" ");tmp--; }
		printf("\t | %d\t | %d\n",t[i].naissance,t[i].etat);
	}
}
